@@ -20,6 +20,7 @@ import abc
 import argparse
 import collections
 import enum
+import json
 import logging
 import pathlib
 import re
@@ -46,7 +47,16 @@ def _mac_address_valid(mac_address: str) -> bool:
     return _MAC_ADDRESS_REGEX.match(mac_address.lower()) is not None
 
 
-_MQTTCallbackUserdata = collections.namedtuple("_MQTTCallbackUserdata", ["retry_count"])
+class _MQTTCallbackUserdata:
+    # pylint: disable=too-few-public-methods; @dataclasses.dataclass when python_requires>=3.7
+    def __init__(
+        self, retry_count: int, device_passwords: typing.Dict[str, str]
+    ) -> None:
+        self.retry_count = retry_count
+        self.device_passwords = device_passwords
+
+    def __eq__(self, other: object) -> bool:
+        return isinstance(other, type(self)) and vars(self) == vars(other)
 
 
 class _MQTTControlledActor(abc.ABC):
@@ -54,7 +64,9 @@ class _MQTTControlledActor(abc.ABC):
     MQTT_STATE_TOPIC_LEVELS = NotImplemented  # type: typing.List[_MQTTTopicLevel]
 
     @abc.abstractmethod
-    def __init__(self, mac_address: str, retry_count: int) -> None:
+    def __init__(
+        self, mac_address: str, retry_count: int, password: typing.Optional[str]
+    ) -> None:
         self._mac_address = mac_address
 
     @abc.abstractmethod
@@ -93,7 +105,12 @@ class _MQTTControlledActor(abc.ABC):
         if not _mac_address_valid(mac_address):
             _LOGGER.warning("invalid mac address %s", mac_address)
             return
-        cls(mac_address=mac_address, retry_count=userdata.retry_count).execute_command(
+        actor = cls(
+            mac_address=mac_address,
+            retry_count=userdata.retry_count,
+            password=userdata.device_passwords.get(mac_address, None),
+        )
+        actor.execute_command(
             mqtt_message_payload=message.payload, mqtt_client=mqtt_client
         )
 
@@ -160,9 +177,15 @@ class _ButtonAutomator(_MQTTControlledActor):
         "state",
     ]
 
-    def __init__(self, mac_address: str, retry_count: int) -> None:
-        self._device = switchbot.Switchbot(mac=mac_address, retry_count=retry_count)
-        super().__init__(mac_address=mac_address, retry_count=retry_count)
+    def __init__(
+        self, mac_address: str, retry_count: int, password: typing.Optional[None]
+    ) -> None:
+        self._device = switchbot.Switchbot(
+            mac=mac_address, password=password, retry_count=retry_count
+        )
+        super().__init__(
+            mac_address=mac_address, retry_count=retry_count, password=password
+        )
 
     def execute_command(
         self, mqtt_message_payload: bytes, mqtt_client: paho.mqtt.client.Client
@@ -205,11 +228,15 @@ class _CurtainMotor(_MQTTControlledActor):
         "state",
     ]
 
-    def __init__(self, mac_address: str, retry_count: int) -> None:
+    def __init__(
+        self, mac_address: str, retry_count: int, password: typing.Optional[None]
+    ) -> None:
         self._device = switchbot.SwitchbotCurtain(
-            mac=mac_address, retry_count=retry_count
+            mac=mac_address, password=password, retry_count=retry_count
+        )
+        super().__init__(
+            mac_address=mac_address, retry_count=retry_count, password=password
         )
-        super().__init__(mac_address=mac_address, retry_count=retry_count)
 
     def execute_command(
         self, mqtt_message_payload: bytes, mqtt_client: paho.mqtt.client.Client
@@ -248,7 +275,7 @@ class _CurtainMotor(_MQTTControlledActor):
 
 def _mqtt_on_connect(
     mqtt_client: paho.mqtt.client.Client,
-    user_data: typing.Any,
+    userdata: _MQTTCallbackUserdata,
     flags: typing.Dict,
     return_code: int,
 ) -> None:
@@ -267,10 +294,13 @@ def _run(
     mqtt_username: typing.Optional[str],
     mqtt_password: typing.Optional[str],
     retry_count: int,
+    device_passwords: typing.Dict[str, str],
 ) -> None:
     # https://pypi.org/project/paho-mqtt/
     mqtt_client = paho.mqtt.client.Client(
-        userdata=_MQTTCallbackUserdata(retry_count=retry_count)
+        userdata=_MQTTCallbackUserdata(
+            retry_count=retry_count, device_passwords=device_passwords
+        )
     )
     mqtt_client.on_connect = _mqtt_on_connect
     _LOGGER.info("connecting to MQTT broker %s:%d", mqtt_host, mqtt_port)
@@ -305,6 +335,14 @@ def _main() -> None:
         dest="mqtt_password_path",
         help="stripping trailing newline",
     )
+    argparser.add_argument(
+        "--device-password-file",
+        type=pathlib.Path,
+        metavar="PATH",
+        dest="device_password_path",
+        help="path to json file mapping mac addresses of switchbot devices to passwords, e.g. "
+        + json.dumps({"11:22:33:44:55:66": "password", "aa:bb:cc:dd:ee:ff": "secret"}),
+    )
     argparser.add_argument(
         "--retries",
         dest="retry_count",
@@ -323,10 +361,15 @@ def _main() -> None:
             mqtt_password = mqtt_password[:-1]
     else:
         mqtt_password = args.mqtt_password
+    if args.device_password_path:
+        device_passwords = json.loads(args.device_password_path.read_text())
+    else:
+        device_passwords = {}
     _run(
         mqtt_host=args.mqtt_host,
         mqtt_port=args.mqtt_port,
         mqtt_username=args.mqtt_username,
         mqtt_password=mqtt_password,
         retry_count=args.retry_count,
+        device_passwords=device_passwords,
     )
